### 大数据相关
### 1. **java8 stream流式API的使用**，加快了处理常见集合的速度，使得在处理数据时更快，更方便简洁  

```java
public Map<String, Object> searchMovies(MovieSearchDto info) {
        Pair<Integer, Integer> pair = RecommendUtils.getStartAndEnd(info.getPage(), info.getSize());
        List<MovieEntity> allMovie = info.getTags().stream()
                .map(t -> getMovies(t, "tag", info.getPage() * info.getSize()))
                .flatMap(Collection::stream)
                .collect(Collectors.toCollection(LinkedList::new));
        if (!StringUtils.isEmpty(info.getContent())) {
            allMovie.addAll(getMovies(info.getContent(), "name", info.getPage() * info.getSize()));
        }
        List<MovieEntity> movieList = allMovie.subList(pair.getLeft(), pair.getRight() <= allMovie.size() ? pair.getRight() : allMovie.size());
        Map<String, Object> result = new HashMap<>(2, 1);
        result.put("total", allMovie.size());
        result.put("movieList", movieList.stream().peek(m -> {
            if (StringUtils.isEmpty(m.getCover())) {
                m.setCover(ServerConstant.DefaultImg);
            }
        }).collect(Collectors.toCollection(LinkedList::new)));
        return result;
    }
```

### 2. 评论和观看记录是大数据，因此采用es处理，部署启动es

```
docker run -d \
--name es \
-p 9200:9200 -p 9300:9300 \
-e "discovery.type=single-node" -e ES_JAVA_OPTS="-Xms200m -Xmx200m" \
-e "xpack.security.enabled=true" \
-v /home/movie/esdata:/usr/share/elasticsearch/data \
elasticsearch:7.5.2 

```
以上方式会报错，查看日志发现是权限问题，因为我们在打数据卷时权限和容器里es，需要的权限是对不上的，所以把容器外数据卷的权限扩大一下，  
保证容器内对外可写就行。  

最好设置密码：  
```
# 进入容器
docker exec -it es /bin/sh
cd bin/
elasticsearch-setup-passwords interactive
# 设置成功后
Changed password for user [apm_system]
Changed password for user [kibana]
Changed password for user [logstash_system]
Changed password for user [beats_system]
Changed password for user [remote_monitoring_user]
Changed password for user [elastic]
```  

### 3. 关于使用es踩了一个大坑
如上使用了最新版的es，并且使用了xpack来进行权限验证，保证数据安全。但是spring boot 2.2.x版本之后才开始支持 es 7.x版本即以上，  
一. 因此首先将spring boot的版本升级到 2.2.0
二. `org.springframework.boot:spring-boot-starter-data-elasticsearch`使用默认的TransportClient是不进行权限校验的，因此链接es失败。  
关于处理方法，网上一大堆，大部分都是复制粘贴，甚至全是错误的。经过自己摸索，解决了此问题，在此做一下汇总，日后处理。
```
1. 首先在maven或者gradle中需要引入xpack依赖包，需要注意在repositories添加https://artifacts.elastic.co/maven源，具体可见build.gradle
网上说需要去除transport依赖，但是我在用时去除依赖会导致某些类加载不到。因此不用去除。
implementation 'org.springframework.boot:spring-boot-starter-data-elasticsearch'
implementation 'org.elasticsearch.client:x-pack-transport:7.5.2'

2.application中添加es配置
spring:
  data:
      elasticsearch:
        repositories:
          enabled: true
    elasticsearch:
      ip: es结点ip，这个是用来注入配置的，具体见ElasticSearchConfig
      rest:
        uris: ip:port # port结点
        username: yourname  # es用户名
        password: yourpassword  # es密码

3. 配置bean，具体见ElasticSearchConfig
```
三. 创建CommentEsRepo，否则是不会自动创建index的，同时还要注意在CommentEs中添加es字段的type
四. 简单的查询可以直接通过CommentEsRepo来实现，复杂的查询需要构造query然后通过ElasticsearchRestTemplate查询即可。  
ElasticsearchRestTemplate是ElasticsearchOperations 接口的另一个实现，其是基于High Level REST Client实现的，所有具有更高的性能  


### Comment评论ES index选型
1. comment中主要包含评论内容，电影id，用户头像，昵称等信息，其中用户头像和昵称是可变的，也就是用户在修改昵称和头像后，在返回给前端时应该是变化后的值  
2. 因此在对comment index选型时有一下几种方案：  

方案 | 对比
---|---
Application-side joins | 应用层连接，独立了文档，但对应用层负载压力增大
Data denormalization | 非规范化数据，通过冗余来扁平化文档
Nested objects | 嵌套对象，文档不可以单独存在，是隐含在父文档中，需要单独建立文档
Parent/child relationships | 父子文档，文档独立，但是需要在同一个分片中，对内存要求高

3. 考虑到服务器性能，以及业务场景，决定采用Data denormalization的方式，即将用户信息扁平处理到Comment index中，但是这样会带来另一个较为棘手的问题  
当更新用户信息时需要对comment文档中的相关记录做更新，目前可以通过logstash来同步，  
但是考虑到配置复杂，且用户信息更新是一个较不频繁的操作，暂时不采用这种方式。  
所以直接在应用层通过异步线程池批量更新的方式去维护  



